/*
  Example illustrating the use of the BlockIP library for stochastics problems.
  The following two-stage problem consists of determining the optimal
capacity investment in various types of power plants so as to meet
next period demands for electricity. The number of power plants an modes 
can change. The next period demand for each of the three modes are to be met. 
There is a budget constraint and also a constraint on the minimum total capacity.
The problem is written in Dual form without spliting (create the .mps for CPLEX)
 */

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <cstring>
#include <vector>
#include "math.h"
#include "BlockIP.h"
#include "ExceptionBlockIP.h"


using namespace std;
void write_options();
void read_options_ElectricDual(int argc, char *argv[], int &index_outfile, int &index_mpsfile, bool &onlymps, bool &quad, int &supplyN, int &Pfacility, int &ClientN, int &numBlocks);
void read_options_BlockIP(BlockIP *bip, int argc, char *argv[]);
//prototipos
void crearL1_MINCAPDual(int techno, int *iL1, int *jL1, int *aL1);
void crearL1_BUDGETDual(int techno, int *iL1, int *jL1, int *aL1);
void crearL2_VarXDual(int techno,int rest1stage, int *iL2, int *jL2, int *aL2);
void crearN2_VarYaDual(int techno,int modesEd, int rest1stage, int *iN2, int *jN2, int *aN2);
void crearN2_VarYbDual(int techno,int modesEd, int rest1stage, int *iN2, int *jN2, int *aN2);
void crearN2_holgurasDual(int techno,int modesEd, int rest1stage, int *iN2, int *jN2, int *aN2);
void crearN2_VarX(int techno, int *iN2, int *jN2, int *aN2);
void crearN2_VarYa(int techno,int modesEd, int *iN2, int *jN2, int *aN2);
void crearN2_VarYb(int techno,int modesEd, int *iN2, int *jN2, int *aN2);
void crearN2_holguras(int techno,int modesEd, int *iN2, int *jN2, int *aN2);

void createInvestment(int techno,double *c1); 
void createProductionCost(int techno, int modesEd, double *c2);

void createRHSDual(int techno, int modesEd, int Blocks,int rhsDualv, double *c1, double *c2, double *rhsDual);
void  createDualCost(int techno, int modesEd, int Blocks, int rest1stage, int costDualv, int VarY, int rest, int LslacksDual, double *costDual);

int main(int argc, char *argv[]) {
  clock_t t0,t1;

  if (argc < 6 ){   //|| argc % 2 != 0) {
    cerr << "usage: ElectricDual need the <number of Supplies>  <number Pfailities> <number of clients> and <number of blocks>  "<< endl
         << "example: ./ElectricDual -ElectricDEsc #Supply #pFacilities #Clients #scenarios -type_comp_dy 0 \n"
         << endl;

    write_options();
    exit(-1);
  }

 // read some options from command line
 int rest1stage=0;//number of constraints first stage
 int techno = 0; //number of available technologies (i) //4
 int modesEd = 0; //number of modes of electicity demand (j) //3
 int Blocks = 0;  // number of blocks (scenarios 1ºstage+2ºscenarios)//3

// for read options Electric
  int index_outfile= -1;
  int index_mpsfile= -1;
  bool onlymps= false;
  bool quad= false;
 
  read_options_ElectricDual(argc,argv,index_outfile,index_mpsfile,onlymps,quad,rest1stage, techno, modesEd, Blocks);
 //for(int i = 0; i < argc; i++) 
  //   cout << argv[i] << " "; 
  // cout << endl; 

 int linking = 1; // if 0 then linking are of type SPLITTING; if 1 then linking are GENERAL matrices 
 int nzN2 = (techno*modesEd)+(techno*modesEd)+(techno*modesEd);
 int nzL1 = techno+techno;
 int nzL2 = techno;
 int VarY = techno*modesEd; //number of variables 2º stage (1 block)
 int restN1=rest1stage+techno+modesEd;
 int VarN1=techno+(techno*modesEd)+techno+rest1stage;
 int restN2=techno+modesEd;
 int VarN2=techno+(techno*modesEd)+techno;
 int VarBlocks=VarN1+(VarN2*(Blocks-1));
 int rest=restN1+(restN2*(Blocks-1));
 int Lslacks = (techno)*(Blocks-1); 
 int LslacksDual=techno;
 int colN2Dual=techno+modesEd+VarY;
 int rhsDualv=((techno*modesEd)*Blocks)+techno;
 int costDualv=rest1stage+((techno+modesEd+VarY)*(Blocks));//variables duales 

cout<<"INICIA Electric"<<endl;

 int *aL1=NULL, *jL1=NULL, *iL1=NULL;
 int *aL2=NULL, *jL2=NULL, *iL2=NULL;
 int *aN2=NULL, *jN2=NULL, *iN2=NULL;
 double *c1=NULL, *c2=NULL, *rhsDual=NULL, *costDual=NULL;

  srand(50); 

cout<<"Blocks= "<<Blocks<<endl;
cout<<"techno= "<<techno<<endl;
cout<<"modesEd= "<<modesEd<<endl;
cout<<"VarBlocks= "<<VarBlocks<<endl;
cout<<"rhsDualv= "<<rhsDualv<<endl;


  iN2=new int[nzN2];
  jN2=new int[nzN2];
  aN2=new int[nzN2];
  iL1=new int[nzL1];
  jL1=new int[nzL1];
  aL1=new int[nzL1];
  iL2=new int[nzL2];
  jL2=new int[nzL2];
  aL2=new int[nzL2];
  c1=new double[techno];
  c2=new double[VarY];
  rhsDual=new double[rhsDualv];
  costDual=new double[costDualv+LslacksDual];

 crearL1_MINCAPDual(techno, iL1, jL1, aL1);
 crearL1_BUDGETDual(techno, iL1, jL1, aL1);
 crearL2_VarXDual(techno,rest1stage, iL2, jL2, aL2);
 crearN2_VarYaDual(techno,modesEd, rest1stage, iN2, jN2,aN2);
 crearN2_VarYbDual(techno,modesEd, rest1stage, iN2, jN2,aN2);
 crearN2_holgurasDual(techno, modesEd, rest1stage, iN2, jN2, aN2);

 createInvestment(techno, c1);
 createProductionCost(techno, modesEd, c2); 

 createRHSDual(techno, modesEd, Blocks,rhsDualv, c1, c2, rhsDual);
 createDualCost(techno, modesEd, Blocks, rest1stage, costDualv, VarY, rest, LslacksDual, costDual);

//for check L1
//for(int i=0; i<nzL1;i++){
//	cout<<"k= "<<i<<" i= "<<iL1[i]<<" j= "<<jL1[i]<<" a="<<aL1[i]<<endl;
//}
//for check L2
//for(int i=0; i<nzL2;i++){
//	cout<<"k= "<<i<<" i= "<<iL2[i]<<" j= "<<jL2[i]<<" a="<<aL2[i]<<endl;
//}
//for check N2
//for(int i=0; i<nzN2;i++){
//	cout<<"k= "<<i<<" i= "<<iN2[i]<<" j= "<<jN2[i]<<" a="<<aN2[i]<<endl;
//}



  MatrixBlockIP *N =  NULL; 
  N = new MatrixBlockIP[Blocks+1];
  int *iW, *jW;
  double *WN; 
  int nz_aux;

//para Q first stage y<=1 and u<=M
	N[0].create_general_matrix_format_ija(0,rest1stage, 0, iW, jW, WN);

	for(int b=1; b<=Blocks; b++){ 
		N[b].create_general_matrix_format_ija(VarY,colN2Dual , nzN2, iW, jW, WN); 
		for (int i = 0; i < nzN2; ++i) {
			iW[i] = iN2[i];
			jW[i] = jN2[i]; 
			WN[i] = aN2[i];
			//cout<<"i= "<<i<<" i1= "<<iW[i]<<"  j= "<<jW[i]<<" = Wa["<<i<<"]= "<<WN[i]<<endl;
		}//for
 	}//forb


  MatrixBlockIP *L =  NULL; 
  L = new MatrixBlockIP[Blocks+1];
  //MatrixBlockIP L;
  int *iL, *jL;
  double *NL;

	L[0].create_general_matrix_format_ija(techno,rest1stage, nzL1, iL, jL, NL); 
		for (int i = 0; i < nzL1; ++i) {
			iL[i] = iL1[i];
			jL[i] = jL1[i]; 
			NL[i] = aL1[i];
			//cout<<"i1= "<<iL1[i]<<"  j= "<<jL1[i]<<" = Wa["<<i<<"]= "<<aL1[i]<<endl;
		}//for

 	for(int b=1; b<=Blocks; b++){ 
		L[b].create_general_matrix_format_ija(techno, colN2Dual, nzL2, iL, jL, NL); 
		for (int i = 0; i < nzL2; ++i) {
			iL[i] = iL2[i];
			jL[i] = jL2[i]; 
			NL[i] = aL2[i];
			//cout<<"i= "<<iL2[i]<<"  j= "<<jL2[i]<<" = Wa["<<i<<"]= "<<aL2[i]<<endl;
		}//for
 	}//forb



//for check N[0]
/*
cout<<"N[0].nz= "<<N[0].nz<<endl;
cout<<"N[1].nz= "<<N[1].nz<<endl;
for(int i=0; i<N[1].nz;i++){
	cout<<"k= "<<i<<" i= "<<iN2[i]<<" j= "<<jN2[i]<<" a="<<aN2[i]<<endl; N.a[i]
	//cout<<" N[1].a["<<i<<"]="<< N[1].a[i]<<endl;
}
*/

//for check N[2]
/*
cout<<"N[2].nz= "<<N[2].nz<<endl;
for(int i=0; i<N[2].nz;i++){
	//cout<<"k= "<<i<<" i= "<<iN2[i]<<" j= "<<jN2[i]<<" a="<<aN2[i]<<endl; N.a[i]
	cout<<" N[2].a["<<i<<"]="<< N[2].a[i]<<endl;
}*/

 BlockIP bip; // declare BlockIP problem
    
   double *cost=NULL, *qcost=NULL, *lb=NULL, *ub=NULL, *lhs=NULL, *rhs=NULL; 
  
 // creation of BlockIP problem
  bip.create_problem(BlockIP::LINEAR, cost, qcost, NULL, NULL, lb, ub, lhs, rhs, Blocks+1, false, N, false, L);//&N(linking==0)


 // fill arrays of data for problem -cost 
	for (int k=0; k<costDualv+LslacksDual; k++){	  
			cost[k] = -costDual[k];	
	} //for  

  //for check
//	for(int i = 0; i < costDualv+LslacksDual; i++){
//		cout<< "cost["<<i<<"]= "<<cost[i]<<endl;
//	}

 // rhs of constraints 
	for (int k=0; k<rhsDualv; k++){  
			rhs[k] = rhsDual[k];
	}//for

 //Lower constraint limits
	for (int i=0; i<rhsDualv; i++){
		lhs[i] = rhs[i];
	}//for

  //for check
//   for(int k=0; k<rhsDualv;k++){    
//     cout<<"rhs["<<k<<"]= "<<rhs[k]<<endl;
//   }

 // upper bound for each variable 
/*
	for (int k=0; k<costDualv; k++){	
		ub[k] = 1.0e+6;
		lb[k] = 0;
		//ub[k] = bip.get_inf();
	}//for 
	for (int i = costDualv; i <costDualv+LslacksDual; ++i) {
	   	ub[i] = 1.0e-8; 
		lb[i] = 0; 
	}//for
*/

    int ubrr=0, ubblkcost=techno+modesEd+VarY; 
	for(int k=0; k<rest1stage;k++){
		ub[k] = 1.0e+6;
		lb[k] = 0;
	}

	for(int k=0; k<ubblkcost*Blocks;k++){
		int ubauxk1=rest1stage;
		if(k<techno+(ubrr*ubblkcost)){ //4
			ub[ubauxk1+k] = 1.0e+11;//para pequeños 1.0e+6;
			lb[ubauxk1+k]  = 0;
		}else{////modosEd demanda
			if(k<(techno+modesEd)+(ubrr*ubblkcost)){
				//ub[ubauxk1+k]  = 1.0e+6;
				//lb[ubauxk1+k]  =  0;
    			ub[ubauxk1+k] = bip.get_inf(); 
				lb[ubauxk1+k]  = -bip.get_inf(); 
			}else{//if Vary
				ub[ubauxk1+k]  = 1.0e+11; //para pequeños 1.0e+6;
				lb[ubauxk1+k]  =  0;
				//cout<<"dcostDual["<<auxk1+k<<"]= "<<costDual[auxk1+k]<<endl;
			}//else
		}//else Blocks
		if(k==(ubblkcost*(ubrr+1))-1){
		//cout<<"cambiio ";
		//cout<<"b= "<<rr<<endl;
			ubrr++;
		}//if
	}//for

	for(int k=0; k<LslacksDual;k++){ 
		int ubauxk2=costDualv; 
			ub[ubauxk2+k]  = 1.0e-6;
			lb[ubauxk2+k]  =  0;
		//cout<<"dcostDual["<<k+auxk2<<"]= "<<costDual[auxk2+k]<<endl;
	}

  //for check
   //for(int k=0; k<costDualv+LslacksDual;k++){    
   //   cout<<"ub["<<k<<"]= "<<ub[k]<<" \t ";
	// cout<<" lb["<<k<<"]= "<<lb[k]<<endl;
   //}

  // change default parameters according to command line values given by user
  read_options_BlockIP(&bip, argc, argv);

cout<<"aqui voy 7 "<<endl;
//  cout << endl << "Write block-MPS format" << endl;
  bip.write_mps("ElDual");
//  cout << endl << "Write BlockIP format" << endl;
//  bip.write_BlockIP_format("ElectricDual");

cout<<"aqui voy 8 "<<endl;

 // set some optionsls
 // bip.set_show_specrad(true); // compute and show estimation of spectral radius

  cout << "BIPnumConst = " << bip.get_km() << endl; //Get the number of constraints without linking constraints
  cout << "BIPnumVaraibles = " << bip.get_kn() << endl;//Get the number of variables without linking constraints
  cout << "BIPnumLinking = " << bip.get_l_link() << endl; //Get the number of linking constraints.
  cout << "numConstOrig = " << bip.get_num_cons() << endl; //Get the number of constraints of the original 
  cout << "numVarOrig = " << bip.get_num_vars() << endl; //Get the number of variables of the original
cout << " optim_gap= " << bip.get_optim_gap() << endl; //Get the optimality gap de BlockIP 
 cout << "init_pcgtol= " << bip.get_init_pcgtol() << endl; //Get  tolerance of the pcg (default in linear problems 1.e-2)
 cout << "red_pcgtol= " << bip.get_red_pcgtol() << endl; //Get if is 1 the tol of pcg don't change in every iteration, number between [0,1] a cada iteración 
cout << "m_pw_prec= " << bip.get_m_pw_prec() << endl;

//solve

  cout << "Solve Problem ** "<<endl; 
 t0= clock();
//  bip.minimize(); // solve problem
 t1= clock();

#ifdef OPENMP
    cout << "Elapsed OpenMP wall clock time: " << bip.get_wclock_time() << endl;
#endif
  cout << "Optimization CPU time: " << (double) (t1-t0)/CLOCKS_PER_SEC << " seconds" << endl;
  cout << "Total IPit= " << bip.get_it() << endl;
  cout << "Total CGit= " << bip.get_cgit() << endl;
  cout << "Primal Objetive = " << bip.get_fobj() << endl;
  cout << "Dual objetive = " << bip.get_dobj() << endl;

////////////////////////

delete[] iL1;
delete[] jL1;
delete[] aL1;
delete[] iL2;
delete[] jL2;
delete[] aL2;
delete[] iN2;
delete[] jN2;
delete[] aN2;
delete[] c1;
delete[] c2;
delete[] rhsDual;
delete[] costDual;


delete [] N;
delete [] L;

  return 0;
}//main

void createDualCost(int techno, int modesEd, int Blocks, int rest1stage, int costDualv, int VarY, int rest, int LslacksDual, double *costDual){
    int rr=0, blkcost=techno+modesEd+VarY; 
	for(int k=0; k<rest1stage;k++){
		if(k==0){ //first constraint MINCAP
			costDual[k]=3*techno;
			//cout<<"costDual["<<k<<"]= "<<costDual[k]<<endl;
		}else{//if
			costDual[k]=-3000*techno;//negativo
			//cout<<"costDual["<<k<<"]= "<<costDual[k]<<endl;					
		}//else k=1
	}

	for(int k=0; k<blkcost*Blocks;k++){
		int auxk1=rest1stage;
		if(k<techno+(rr*blkcost)){ //4
			costDual[auxk1+k]=0;
			//cout<<"costDual["<<auxk1+k<<"]= "<<costDual[auxk1+k]<<endl;
		}else{////modosEd demanda
			if(k<(techno+modesEd)+(rr*blkcost)){
				costDual[auxk1+k]=(rand() % 10 + 2);
				//cout<<"dcostDual["<<auxk1+k<<"]= "<<costDual[auxk1+k]<<endl;
			}else{//if Vary
				costDual[auxk1+k]=0;
				//cout<<"dcostDual["<<auxk1+k<<"]= "<<costDual[auxk1+k]<<endl;
			}//else
		}//else Blocks
		if(k==(blkcost*(rr+1))-1){
		//cout<<"cambiio ";
		//cout<<"b= "<<rr<<endl;
			rr++;
			
		}//if
	}//for

	for(int k=0; k<LslacksDual;k++){ 
		int auxk2=costDualv; 
		costDual[auxk2+k]=0;  
		//cout<<"dcostDual["<<k+auxk2<<"]= "<<costDual[auxk2+k]<<endl;
	}

}//void
void createRHSDual(int techno, int modesEd, int Blocks, int rhsDualv, double *c1, double *c2, double *rhsDual){

int auxk=0, auxc2=0, auxc1=0, vblk=techno*modesEd, rr=1; 
	for(int k=0; k<(rhsDualv-techno);k++){
		if(k<vblk*rr){
			rhsDual[k]=c2[auxc2]*(1.0/(double)Blocks);;
			auxc2++;
			if(k==(vblk*rr)-1){
				auxc2=0;
				rr++;
			}
		}
	auxk++;
	}//for
	for(int k=auxk; k<auxk+techno;k++){  
		rhsDual[k]=c1[auxc1];
		auxc1++;
	}
 //for check	
// for(int k=0; k<rhsDualv;k++){    
// 	cout<<"rhsDual["<<k<<"]= "<<rhsDual[k]<<endl;
// }
}//void

void createInvestment(int techno, double *c1){ 
	for(int k=0; k<techno;k++){ 
		c1[k]=(rand() % 20 + 5); 
		//cout<<"c1["<<k<<"]= "<<c1[k]<<endl;
	}//for
}//void

void createProductionCost(int techno, int modesEd, double *c2){ 
   double *auxt=NULL, *auxmEd=NULL;
   int auxk=0;
    auxt=new double[techno];
	auxmEd=new double[modesEd]; 

	for(int k=0; k<techno;k++){ 
		auxt[k]=(rand() % (500-200)) / 100.0;
		//cout<<"auxt["<<k<<"]= "<<auxt[k]<<endl;
	}

	for(int k=0; k<modesEd;k++){ 
		auxmEd[k]=(rand() % 8 + 0);
		//cout<<"auxmEd["<<k<<"]= "<<auxmEd[k]<<endl;
	}
	
	for(int i=0; i<modesEd;i++){ 
		for(int j=0; j<techno;j++){
		c2[auxk]=auxt[j]*auxmEd[i];
		//cout<<"c2["<<auxk<<"]= "<<c2[auxk]<<endl;
		auxk++;
		}//for
	}//for

	//for(int k=0; k<techno*modesEd;k++){ 
		//cout<<"c2a["<<k<<"]= "<<c2[k]<<endl;
	//}

	delete[] auxt;
	delete[] auxmEd;
}//void

void crearL1_MINCAPDual(int techno, int *iL1, int *jL1, int *aL1){
  int aux1i=0, aux1j=0;
	for(int k=0; k<techno;k++){ 
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iL1[k]=aux1i;
		jL1[k]=aux1j;
		aL1[k]=1;
		aux1i++;
	}//for
}//void

void crearL1_BUDGETDual(int techno, int *iL1, int *jL1, int *aL1){
  int aux1i=0, aux1j=1;
	for(int k=techno; k<techno+techno;k++){ //8
		iL1[k]=aux1i;
		jL1[k]=aux1j;
		aL1[k]=-(rand() % 100 + 5); //negativo var><
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" a = "<<aN1[k]<<" ->"<<k<<endl;
		aux1i++;
	}//for
}//void

void crearL2_VarXDual(int techno, int rest1stage, int *iL2, int *jL2, int *aL2){
  int aux1j=0, aux1i=0;
	for(int k=0; k<techno;k++){ //8
		iL2[k]=aux1i;
		jL2[k]=aux1j;
		aL2[k]=1;//positivo
		cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		aux1j++;
		aux1i++;
	}//for
}//void

void crearN2_VarYaDual(int techno,int modesEd, int rest1stage, int *iN2, int *jN2, int *aN2){
  int rr=1, aux1j=0, aux1i=0;
	for(int k=0; k<techno*modesEd;k++){ //16
	//for(int k=(techno*rest1stage)+techno; k<(techno*rest1stage)+techno+(techno*modesEd);k++){ //16
		//cout<<" ->"<<k<<" i= "<<aux1i<<" j= "<<aux1j<<endl;
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=-1;//negativo
		//cout<<" ->"<<k<<" i= "<<aux1i<<" j= "<<aux1j<<endl;
		aux1i++;
		if(k==((modesEd*rr)-1)){ //cada modesED
			//cout<<"cambio "<<((modesEd*rr)-1) <<endl;//4 cambios=
			aux1j++;
			//aux1j=rr;
			rr++;
		}//if
	}//for
}//void

void crearN2_VarYbDual(int techno,int modesEd, int rest1stage, int *iN2, int *jN2, int *aN2){
  int rr=1, aux1j=techno, aux1i=0, auxk=techno*modesEd;
for(int k=auxk; k<auxk+(techno*modesEd);k++){ 	
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=1;
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		aux1i=aux1i+modesEd;
		if(k==((techno*rr)-1)+auxk){ //cada 
			//cout<<"cambio " <<endl;//4 cambios=
			aux1j++;
			aux1i=rr;
			rr++;
		}//if
	}//for
}//void

void crearN2_holgurasDual(int techno,int modesEd, int rest1stage, int *iN2, int *jN2, int *aN2){
  int aux1i=0, aux1j=techno+modesEd, auxk=(techno*modesEd)+(techno*modesEd);
	for(int k=auxk; k<auxk+(techno*modesEd); k++){ 
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=1;
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		aux1i++;
		aux1j++;
	}//for
}//void


void crearN2_VarX(int techno, int *iN2, int *jN2, int *aN2){
  int aux1i=0, aux1j=0;
	for(int k=0; k<techno;k++){ //8
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=-1;
		aux1j++;
		aux1i++;
	}//for
}//void


void crearN2_VarYa(int techno,int modesEd, int *iN2, int *jN2, int *aN2){
  int rr=1, aux1i=0, aux1j=techno;
	for(int k=techno; k<techno+(techno*modesEd);k++){ 
		//cout<<" ->"<<k<<" i= "<<aux1i<<" j= "<<aux1j<<endl;
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=1;
		aux1j++;
		if(k==((modesEd*rr)-1)+techno){ //cada modesED
			//cout<<"cambio "<<((modesEd*rr)-1) <<endl;//4 cambios=
			aux1i++;
			rr++;
		}//if
	}//for
}//void

void crearN2_VarYb(int techno,int modesEd, int *iN2, int *jN2, int *aN2){
  int rr=1, aux1i=techno, aux1j=techno, auxk=techno+(techno*modesEd);
for(int k=auxk; k<auxk+(techno*modesEd);k++){ 	
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=1;
		aux1j=aux1j+modesEd;
		if(k==((techno*rr)-1)+auxk){ //cada Pfacitity
			//cout<<"cambio " <<endl;//4 cambios=
			aux1i++;
			aux1j=techno+rr;
			rr++;
		}//if
	}//for
}//void

void crearN2_holguras(int techno,int modesEd, int *iN2, int *jN2, int *aN2){
  int aux1i=0, aux1j=techno+(techno*modesEd), auxk=techno+(techno*modesEd)+(techno*modesEd);
	for(int k=auxk; k<auxk+techno; k++){ //6
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=1;
		aux1i++;
		aux1j++;
	}//for
}//void


/////////////////////////////////////////////////////////////////////////////////////////////////////////







void write_options() {
  cout <<
  "  -ElectricDEsc {numberFirstConstraints} {numbertechno} {numbermodesEd} {Escenarios}(required) \n"
  "  -out {name}\n"
  "    Output file for optimal variables (if not provided, solution is not reported)\n"
  "  -mps {name}\n"
  "    If name is provided, an mps format of the problem is written to filename 'name'\n"
  "  -only_mps {name}\n"
  "    If name is provided, the program stops after  writing mps format (problem is not solved)\n"
  "  -quad {y/n}\n"
  "    If y costs are considered quadratic; otherwise linear (deefault is linear)\n"
  "  -inf {value}\n"
  "    Set infinity value to be considered\n"
  "  -m_pw_prec {value}\n"
  "    Set number of terms used as preconditioner of the power series expansion of (D-C'*B^{-1}*B)^{-1}\n"
  "  -sigma {value}\n"
  "    Set reduction of the centrality parameter at each IP iteration\n"
  "  -rho {value}\n"
  "    Set reduction of the step-length for the primal and dual variables at each IP iteration\n"
  "  -optim_gap {value}\n"
  "    Set optimality gap tolerance\n"
  "  -optim_pfeas {value}\n"
  "    Set primal feasibility tolerance\n"
  "  -optim_dfeas {value}\n"
  "    Set dual feasibility tolerance\n"
  "  -output_freq {value}\n"
  "    Set output information lines will be printed each output_freq IP iterations\n"
  "  -output {value}\n"
  "    Set type of output. Can take the values: 0= NONE, 1= SCREEN, 2= FILE, 3=BOTH\n"
  "  -maxiter {value}\n"
  "    Set maximum number of IP iterations\n"
  "  -min_pcgtol {value}\n"
  "    Set minimum tolerance for the conjugate gradient\n"
  "  -red_pcgtol {value}\n"
  "    Set reduction of pcg_tol at each IP iteration\n"
  "  -init_pcgtol {value}\n"
  "    Set initial tolerance for the conjugate gradient (by default PCG_TOL_LIN if linear problem, PCG_TOL_QUAD if quadratic or -convex- nonlinear)\n"
  "  -type_start_point {value}\n"
  "    Set how starting point is computed. Can take the values: 0= SIMPLE, 1= QUAD_EQCONS_PROB\n"
  "  -type_comp_dy {value}\n"
  "    Set how dy direction is computed. Can take the values: 0= CHOL_PWRS_PCG, 1= FULL_CHOL, 2= HYBRID_PCG, 3= CHOL_THETA0_PCG, 4= PWRS_THETA0_PCG, 5=THETA0_PWRS_PCG, 6=AUT_PWRSTHETA0_PCG, 7=DYN_PWRSTHETA0_PCG\n"
  "  -type_direction {value}\n"
  "    Set which direction is computed. Can take the values: 0= NEWTON, 1= SECOND_ORDER, 2= AUTOMATIC\n"
  "  -deactivateLnk {value}\n"
  "    Set flag on deactivation of linking\n"
  "  -type_reg {value}\n"
  "    Set type of regularization performed. Can take the values: 0=NO_REG, 1= QUAD_REG, 2= PROX_REG\n" 
  "  -factor_reg {value}\n"
  "    Set initial value for regularization term\n" 
  "  -show_specrad {y/n}\n"
  "    Whether estimation of spectral radius by Ritz values is shown at each IP iteration\n"
  "  -show_princ_angles {y/n}\n"
  "    Whether average principal angles between L and N are shown when computed\n"
  "  -threshold_angle {value}\n"
  "    Set threshold principal angle to switch between PWRS and THETA0 preconditioners\n"
  "  -threshold_specrad {value}\n"
  "    Set threshold spectral radius to switch between PWRS and THETA0 preconditioners\n"
  "  -it_Theta0PWRS {value}\n"
  "    Set number of iterations between checks to switch between PWRS and THETA0\n"
  "  -type_comp_angle {value}\n"
  "    Set how principal angles are computed. Possible values are: 0= LINEARCOMB, 1= AVERAGE\n"
  "  -gap_changeChol {value}\n"
  "    Change to Cholesky if duality gap below this value\n"
  "  -zero_pivots {value}\n"
  "    Max number of zero pivots allowed in Cholesky factorization\n"
  "  -show_zero_pivots {y/n}\n"
  "    Show_zero_pivots at each IP iteration\n"
  "  -stop_if_PCG_fails {y/n}\n"
  "    If PCG fails stop, do not switch to Cholesky, useful if instance is large\n"
  "  -freevars {0/1}\n"
  "    How to deal with free variables: 0=SPLIT , 1=REGULARIZE\n"
  "  -threads n \n"
  "    Set number of threads (by default 1; if n<=1, 1; otherwise min{n,max number of threads})\n"
  << endl;
}//end void write_options()

void read_options_ElectricDual(int argc, char *argv[], int &index_outfile, int &index_mpsfile, bool &onlymps, bool &quad, int &rest1stage, int &techno, int &modesEd, int &Blocks) {
  char filename[512];
  char c;
//  int supplyNa, Pfacilitya, ClientNa, numBloksa;
  for (int argIndex = 1; argIndex < argc; argIndex += 2) {
    if (!strcmp(argv[argIndex], "-ElectricDEsc") && sscanf(argv[argIndex+1], "%i", &rest1stage) && sscanf(argv[argIndex+2], "%i", &techno) &&  sscanf(argv[argIndex+3], "%i", &modesEd) && sscanf(argv[argIndex+4], "%i", &Blocks) ){
	argIndex +=3;
        index_outfile= argIndex+1;
	}//if !strcmp(argv[argIndex]
    else if (!strcmp(argv[argIndex], "-out") && sscanf(argv[argIndex+1], "%s", &filename) )
      index_outfile= argIndex+1;
    else if (!strcmp(argv[argIndex], "-mps") && sscanf(argv[argIndex+1], "%s", &filename) )
      index_mpsfile= argIndex+1;
    else if (!strcmp(argv[argIndex], "-only_mps") && sscanf(argv[argIndex+1], "%s", &filename) ) {
      index_mpsfile= argIndex+1;
      onlymps= true;
    }
    else if (!strcmp(argv[argIndex], "-quad") && sscanf(argv[argIndex+1], "%c", &c) ) {
      quad= (c== 'y');
    }
    else if (!strcmp(argv[argIndex], "-inf") ||
             !strcmp(argv[argIndex], "-m_pw_prec") ||
             !strcmp(argv[argIndex], "-sigma") ||
             !strcmp(argv[argIndex], "-rho") ||
             !strcmp(argv[argIndex], "-optim_gap") ||
             !strcmp(argv[argIndex], "-optim_pfeas") ||
             !strcmp(argv[argIndex], "-optim_dfeas") ||
             !strcmp(argv[argIndex], "-output_freq") ||
             !strcmp(argv[argIndex], "-output") ||
             !strcmp(argv[argIndex], "-maxiter") ||
             !strcmp(argv[argIndex], "-min_pcgtol") ||
             !strcmp(argv[argIndex], "-red_pcgtol") ||
             !strcmp(argv[argIndex], "-init_pcgtol") ||
             !strcmp(argv[argIndex], "-type_start_point") ||
             !strcmp(argv[argIndex], "-type_comp_dy") ||
             !strcmp(argv[argIndex], "-type_direction") ||
             !strcmp(argv[argIndex], "-deactivateLnk") ||
             !strcmp(argv[argIndex], "-type_reg") ||
             !strcmp(argv[argIndex], "-factor_reg") ||
             !strcmp(argv[argIndex], "-show_specrad") ||
             !strcmp(argv[argIndex], "-show_princ_angles") ||
             !strcmp(argv[argIndex], "-threshold_angle") ||
             !strcmp(argv[argIndex], "-threshold_specrad") ||
             !strcmp(argv[argIndex], "-it_Theta0PWRS") ||
             !strcmp(argv[argIndex], "-type_comp_angle") ||
             !strcmp(argv[argIndex], "-gap_changeChol") ||
             !strcmp(argv[argIndex], "-zero_pivots") ||
             !strcmp(argv[argIndex], "-show_zero_pivots") ||
             !strcmp(argv[argIndex], "-stop_if_PCG_fails") ||
		     !strcmp(argv[argIndex], "-freevars") ||
             !strcmp(argv[argIndex], "-threads") 
             )
      // do nothing 
      {}
    else {
      cerr << "usage: Electric  {options}\n";
      write_options();
      exit(-1);
    }
  }
}
 


void read_options_BlockIP(BlockIP *bip, int argc, char *argv[]) {
  double d;
  int i;
  char c;
  for (int argIndex = 6; argIndex < argc; argIndex += 2) {
    if (!strcmp(argv[argIndex], "-inf") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_inf(d);
    else if (!strcmp(argv[argIndex], "-m_pw_prec") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_m_pw_prec(i);
    else if (!strcmp(argv[argIndex], "-sigma") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_sigma(d);
    else if (!strcmp(argv[argIndex], "-rho") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_rho(d);
    else if (!strcmp(argv[argIndex], "-optim_gap") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_optim_gap(d);
    else if (!strcmp(argv[argIndex], "-optim_pfeas") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_optim_pfeas(d);
    else if (!strcmp(argv[argIndex], "-optim_dfeas") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_optim_dfeas(d);
    else if (!strcmp(argv[argIndex], "-output_freq") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_output_freq(i);
    else if (!strcmp(argv[argIndex], "-output") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_output((BlockIP::OUTPUT)i);
    else if (!strcmp(argv[argIndex], "-maxiter") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_maxiter(i);
    else if (!strcmp(argv[argIndex], "-min_pcgtol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_min_pcgtol(d);
    else if (!strcmp(argv[argIndex], "-red_pcgtol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_red_pcgtol(d);
    else if (!strcmp(argv[argIndex], "-init_pcgtol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_init_pcgtol(d);
    else if (!strcmp(argv[argIndex], "-type_start_point") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_start_point((BlockIP::TYPE_START_POINT)i);
    else if (!strcmp(argv[argIndex], "-type_comp_dy") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_comp_dy((BlockIP::TYPE_COMP_DY)i);
    else if (!strcmp(argv[argIndex], "-type_direction") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_direction((BlockIP::TYPE_DIRECTION)i);
    else if (!strcmp(argv[argIndex], "-deactivateLnk") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_deactivateLnk(i);
    else if (!strcmp(argv[argIndex], "-type_reg") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_reg((BlockIP::TYPE_REG)i);
    else if (!strcmp(argv[argIndex], "-factor_reg") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_factor_reg(d);
    else if (!strcmp(argv[argIndex], "-show_specrad") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_show_specrad(c=='y');
    else if (!strcmp(argv[argIndex], "-show_princ_angles") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_show_princ_angles(c=='y');
    else if (!strcmp(argv[argIndex], "-threshold_angle") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_threshold_angle(d);
    else if (!strcmp(argv[argIndex], "-threshold_specrad") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_threshold_specrad(d);
    else if (!strcmp(argv[argIndex], "-it_Theta0PWRS") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_it_Theta0PWRS(i);
    else if (!strcmp(argv[argIndex], "-type_comp_angle") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_comp_angle((BlockIP::TYPE_COMP_ANGLE)i);
    else if (!strcmp(argv[argIndex], "-gap_changeChol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_gap_change_to_Chol(d);
    else if (!strcmp(argv[argIndex], "-zero_pivots") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_allowed_zero_pivots(i);
    else if (!strcmp(argv[argIndex], "-show_zero_pivots") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_show_zero_pivots(c=='y');
    else if (!strcmp(argv[argIndex], "-stop_if_PCG_fails") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_stop_if_PCG_fails(c=='y');
    else if (!strcmp(argv[argIndex], "-freevars") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_deal_freevars((BlockIP::TYPE_DEAL_FREEVARS)i);
    else if (!strcmp(argv[argIndex], "-threads") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_num_threads(i);
    else if (!strcmp(argv[argIndex], "-ElectricDEsc")||
	     !strcmp(argv[argIndex], "-out")||
	     !strcmp(argv[argIndex], "-mps")||
	     !strcmp(argv[argIndex], "-only_mps") ||
	     !strcmp(argv[argIndex], "-quad")
	     )
      // do nothing
      {}
    else {
      cerr << "usage: Bip_BlockIP {options}\n";
      write_options();
      exit(-1);
    }
  }
}//end read_options_BlockIP(BlockIP *bip, int argc, char *argv[])

